#include "caur.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <sys/wait.h>

int setup_cache_directory(char *cache_dir, size_t size) {
    const char *home = getenv("HOME");
    if (home == NULL) {
        fprintf(stderr, "Could not determine the user's home directory.\n");
        return 1;
    }

    snprintf(cache_dir, size, "%s/.config/caur", home);

    struct stat st = {0};
    if (stat(cache_dir, &st) == -1) {
        if (mkdir(cache_dir, 0755) == -1) {
            fprintf(stderr, "Failed to create cache directory: %s\n", strerror(errno));
            return 1;
        } else {
            printf("Cache directory created: %s\n", cache_dir);
        }
    } else {
        printf("Cache directory already exists: %s\n", cache_dir);
    }

    return 0;
}

int build_package(const char *package_name) {
    char cache_dir[1024];
    char tar_file[1024];
    char unpack_dir[1024];
    char build_command[1024];

    int ret;

    // Set up the cache directory
    if (setup_cache_directory(cache_dir, sizeof(cache_dir)) != 0) {
        return 1;
    }

    // Path to the downloaded .tar.gz file
    ret = snprintf(tar_file, sizeof(tar_file), "%s/%s.tar.gz", cache_dir, package_name);
    if ((size_t)ret >= sizeof(tar_file)) {
        fprintf(stderr, "Warning: tar_file path was truncated.\n");
        return 1;
    }

    // Check if the tar.gz file exists
    struct stat st;
    if (stat(tar_file, &st) != 0) {
        fprintf(stderr, "Error: The file %s does not exist. Please check the download process.\n", tar_file);
        return 1;
    }

    // Create directory to unpack the files into
    ret = snprintf(unpack_dir, sizeof(unpack_dir), "%s/%s", cache_dir, package_name);
    if ((size_t)ret >= sizeof(unpack_dir)) {
        fprintf(stderr, "Warning: unpack_dir path was truncated.\n");
        return 1;
    }
    mkdir(unpack_dir, 0755);

    // Unpack the downloaded .tar.gz file
    if (unpack_tar_gz(tar_file, unpack_dir) != 0) {
        return 1;
    }

    // Change directory to unpacked location and build the package
    ret = snprintf(build_command, sizeof(build_command), "cd %s && makepkg -si", unpack_dir);
    if ((size_t)ret >= sizeof(build_command)) {
        fprintf(stderr, "Warning: build_command was truncated.\n");
        return 1;
    }

    int result = system(build_command);
    if (result == -1) {
        fprintf(stderr, "Failed to execute build command\n");
        return 1;
    } else if (WIFEXITED(result) && WEXITSTATUS(result) != 0) {
        fprintf(stderr, "Build failed with exit status %d.\n", WEXITSTATUS(result));
        return 1;
    }

    printf("Package %s built and installed successfully.\n", package_name);
    return 0;
}

